#!/usr/bin/ruby

require 'minitest/autorun'
require 'foreman_vm/allocator'

class TestCluster < Minitest::Test

  # A mockup of the class we want to write
  # 
  # Goals:
  #   * Allocate new VMs on the hosts with the most free memory
  #   * Do not allocate all instances of a VM type
  #     on the same hypervisor. Example: if there is only one hypervisor
  #     with a VM named 'web1',  do not create a VM named 'web2' here.
  #
  #   * (Stretch goal) Try to spread out VMs across multiple hosts where possible
  #     

  # Try to add a host
  def test_add_host
    alloc = ForemanAP::Allocator.new
    assert alloc.add_host('host1', 1024, ['guest1', 'guest2'])
  end
  
  # Try to add a guest
  def test_alloc_guest
    alloc = ForemanAP::Allocator.new
    assert(alloc.add_host('host1', 1024, ['guest1', 'guest2']))
    assert(alloc.add_guest('guest3', 512))
  end

  # Try to add a guest exceeds free memory of host
  def test_unable_to_add_guest
    alloc = ForemanAP::Allocator.new
    assert(alloc.add_host('host1', 1024, ['guest1', 'guest2']))
    assert_nil(alloc.add_guest('guest3', 4096))
  end

  # Verify guest is put on host with most free memory
  def test_free_memory
    alloc = ForemanAP::Allocator.new 
    assert(alloc.add_host('host1', 2048, ['a1', 'a2']))
    assert(alloc.add_host('host2', 1024, ['a1', 'a2']))
    assert_equal('host1', alloc.add_guest('b1', 512))
  end

  # Try to add guest that already exists
  def test_duplicate_guest
    alloc = ForemanAP::Allocator.new
    assert(alloc.add_host('host1', 2048, ['guest1', 'guest2']))
    assert_nil(alloc.add_guest('guest2', 512))
  end

  # Ensure VM Instance of similar type is spread out to different hypervisors
  def test_affinity
    alloc = ForemanAP::Allocator.new
    assert(alloc.add_host('host1', 2048, ['guest1', 'guest2']))
    assert(alloc.add_host('host2', 4096, ['guest1', 'web1']))
    assert_equal('host1', alloc.add_guest('web2', 512))
  end
end
